/* Ansteurung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus *
 * Datum: 10.10.2010												*/

#ifndef LCDLIB_H
#define LCDLIB_H


/* Verwendete Taktfrequenz in Hz */


#ifndef F_CPU
#define F_CPU 8000000
#endif


/* Pinbelegung fr das LCD */

// LCD DB4-DB7 <--> PORTD Bit PD0-PD3
#define LCD_PORT		PORTD
#define LCD_DDR			DDRD
#define LCD_DB			PD0

// LCD RS <--> PORTD Bit PD4	(RS: 0=Data, I=Command)
#define LCD_RS			PD4

// LCD EN <--> PORTD Bit PD5	(EN: 1-Impuls fr Daten)
#define LCD_EN			PD5


/* LCD Ausfhrungszeiten */

#define LCD_BOOTUP_MS			15
#define LCD_ENABLE_US			1
#define LCD_WRITEDATA_US		46
#define LCD_COMMAND_US			42

#define LCD_SOFT_RESET_MS1		5
#define LCD_SOFT_RESET_MS2		1
#define LCD_SOFT_RESET_MS3		1
#define LCD_SET_4BITMODE_MS		5

#define LCD_CLEAR_DISPLAY_MS	2
#define	LCD_CURSOR_HOME_MS		2


/* Zeilendefinition des verwendeten LCD */

#define LCD_DDADR_LINE1			0x00
#define LCD_DDADR_LINE2			0x40
#define LCD_DDADR_LINE3			0x10
#define LCD_DDADR_LINE4			0x50


// Initialisierung: ganz am Anfang
void lcd_init( void );

// LCD loeschen
void lcd_clear( void );

// Cursor in die 1. Zeile, 0-te Spalte
void lcd_home( void );

// Cursor an eine beliebige Position
void lcd_setcursor( uint8_t spalte, uint8_t zeile );

// Ausgabe eines einzelnen Zeichens an der aktuellen Cursorposition
void lcd_data( uint8_t data );

// Ausgabe eines Strings an der aktuellen Cursorposition
void lcd_string( const char *data );

void copyStringtoLCD(const uint8_t *FlashLoc, uint8_t x, uint8_t y);

// Ausgabe eines Strings an einer bestimmten Cursorposition
void lcd_string_xy( const char *data, uint8_t x, uint8_t y );

// Definition eines benutzerdefinierten Sonderzeichens
// data muss auf ein Array[5] mit den Spaltencodes des zu definierten Zeichens zeigen
void lcd_generatechar( uint8_t code, const uint8_t *data );

// Ausgabe eines Kommandos an das LCD
void lcd_command( uint8_t data );


/* LCD Befehle und Argumente */

// Clear Display
#define LCD_CLEAR_DISPLAY		0x01

// Cursor Home
#define LCD_CURSOR_HOME			0x02

// Set Entry Mode
#define LCD_SET_ENTRY			0x04

#define LCD_ENTRY_DECREASE		0x00
#define LCD_ENTRY_INCREASE		0x02
#define LCD_ENTRY_NOSHIFT		0x00
#define LCD_ENTRY_SHIFT			0x01

// Set Display 
#define LCD_SET_DISPLAY         0x08
 
#define LCD_DISPLAY_OFF         0x00
#define LCD_DISPLAY_ON          0x04
#define LCD_CURSOR_OFF          0x00
#define LCD_CURSOR_ON           0x02
#define LCD_BLINKING_OFF        0x00
#define LCD_BLINKING_ON         0x01
 
// Set Shift
#define LCD_SET_SHIFT           0x10
 
#define LCD_CURSOR_MOVE         0x00
#define LCD_DISPLAY_SHIFT       0x08
#define LCD_SHIFT_LEFT          0x00
#define LCD_SHIFT_RIGHT         0x04
 
// Set Function
#define LCD_SET_FUNCTION        0x20
 
#define LCD_FUNCTION_4BIT       0x00
#define LCD_FUNCTION_8BIT       0x10
#define LCD_FUNCTION_1LINE      0x00
#define LCD_FUNCTION_2LINE      0x08
#define LCD_FUNCTION_5X7        0x00
#define LCD_FUNCTION_5X10       0x04
 
#define LCD_SOFT_RESET          0x30
 
// Set CG RAM Address
#define LCD_SET_CGADR           0x40
 
#define LCD_GC_CHAR0            0
#define LCD_GC_CHAR1            1
#define LCD_GC_CHAR2            2
#define LCD_GC_CHAR3            3
#define LCD_GC_CHAR4            4
#define LCD_GC_CHAR5            5
#define LCD_GC_CHAR6            6
#define LCD_GC_CHAR7            7
 
// Set DD RAM Address
#define LCD_SET_DDADR           0x80
 
#endif
